from uctypes import struct, BF_POS, BF_LEN, UINT32, BFUINT32

DMA_BASE = 0x50000000

DMA_CTRL_TRIG_FIELD = {
    "AHB_ERROR":   31<<BF_POS | 1<<BF_LEN | BFUINT32,
    "READ_ERROR":  30<<BF_POS | 1<<BF_LEN | BFUINT32,
    "WRITE_ERROR": 29<<BF_POS | 1<<BF_LEN | BFUINT32,
    "BUSY":        24<<BF_POS | 1<<BF_LEN | BFUINT32,
    "SNIFF_EN":    23<<BF_POS | 1<<BF_LEN | BFUINT32,
    "BSWAP":       22<<BF_POS | 1<<BF_LEN | BFUINT32,
    "IRQ_QUIET":   21<<BF_POS | 1<<BF_LEN | BFUINT32,
    "TREQ_SEL":    15<<BF_POS | 6<<BF_LEN | BFUINT32,
    "CHAIN_TO":    11<<BF_POS | 4<<BF_LEN | BFUINT32,
    "RING_SEL":    10<<BF_POS | 1<<BF_LEN | BFUINT32,
    "RING_SIZE":    6<<BF_POS | 4<<BF_LEN | BFUINT32,
    "INCR_WRITE":   5<<BF_POS | 1<<BF_LEN | BFUINT32,
    "INCR_READ":    4<<BF_POS | 1<<BF_LEN | BFUINT32,
    "DATA_SIZE":    2<<BF_POS | 2<<BF_LEN | BFUINT32,
    "HIGH_PRIORITY":1<<BF_POS | 1<<BF_LEN | BFUINT32,
    "EN":           0<<BF_POS | 1<<BF_LEN | BFUINT32
    }

DMA_CHANNEL_REGS = {
    "READ_ADDR_REG":       0x000|UINT32,
    "WRITE_ADDR_REG":      0x004|UINT32,
    "TRANS_COUNT_REG":     0x008|UINT32,
    "CTRL_TRIG_REG":       0x00c|UINT32,
    "CTRL_TRIG":          (0x00c,DMA_CTRL_TRIG_FIELD), #starts the channel
#     "AL1_CTRL":                 0x010|UINT32,
#     "AL1_READ_ADDR":            0x014|UINT32,
#     "AL1_WRITE_ADDR":           0x018|UINT32,
    "TRANS_COUNT_TRIG":     0x01c|UINT32, #same as TRANS_COUNT, starts the channel
#     "AL2_CTRL":                 0x020|UINT32,
#     "AL2_TRANS_COUNT":          0x024|UINT32,
#     "AL2_READ_ADDR":            0x028|UINT32,
    "WRITE_ADDR_TRIG":      0x02c|UINT32, #same as WRITE_ADDR, starts the channel
#     "AL3_CTRL":                 0x030|UINT32,
#     "AL3_WRITE_ADDR":           0x034|UINT32,
#     "AL3_TRANS_COUNT":          0x038|UINT32,
    "READ_ADDR_TRIG":       0x03c|UINT32, #same as READ_ADDR, starts the channel
    }

DMA_REGS = {
    #192 registers represented in DMA_CHAN_REGS * channel count , 16 registers each
    "INTR":                         0x400|UINT32,
    "INTE0":                        0x404|UINT32,
    "INTF0":                        0x408|UINT32,
    "INTS0":                        0x40c|UINT32,
    "INTE1":                        0x414|UINT32,
    "INTF1":                        0x418|UINT32,
    "INTS1":                        0x41c|UINT32,
    "TIMER0":                       0x420|UINT32,
    "TIMER1":                       0x424|UINT32,
    "TIMER2":                       0x428|UINT32,
    "TIMER3":                       0x42c|UINT32,
    "MULTI_CHAN_TRIGGER":           0x430|UINT32,
    "SNIFF_CTRL":                   0x434|UINT32,
    "SNIFF_DATA":                   0x438|UINT32,
    "FIFO_LEVELS":                  0x440|UINT32,
    "CHAN_ABORT":                   0x444|UINT32,
    "N_CHANNELS":                   0x448|UINT32,
    "CH0_DBG_CTDREQ":               0x800|UINT32,
    "CH0_DBG_TCR":                  0x804|UINT32, #reload value for TRANS_COUNT_REG
    "CH1_DBG_CTDREQ":               0x840|UINT32,
    "CH1_DBG_TCR":                  0x844|UINT32,
    "CH2_DBG_CTDREQ":               0x880|UINT32,
    "CH2_DBG_TCR":                  0x884|UINT32,
    "CH3_DBG_CTDREQ":               0x8c0|UINT32,
    "CH3_DBG_TCR":                  0x8c4|UINT32,
    "CH4_DBG_CTDREQ":               0x900|UINT32,
    "CH4_DBG_TCR":                  0x904|UINT32,
    "CH5_DBG_CTDREQ":               0x940|UINT32,
    "CH5_DBG_TCR":                  0x944|UINT32,
    "CH6_DBG_CTDREQ":               0x980|UINT32,
    "CH6_DBG_TCR":                  0x984|UINT32,
    "CH7_DBG_CTDREQ":               0x9c0|UINT32,
    "CH7_DBG_TCR":                  0x9c4|UINT32,
    "CH8_DBG_CTDREQ":               0xa00|UINT32,
    "CH8_DBG_TCR":                  0xa04|UINT32,
    "CH9_DBG_CTDREQ":               0xa40|UINT32,
    "CH9_DBG_TCR":                  0xa44|UINT32,
    "CH10_DBG_CTDREQ":              0xa80|UINT32,
    "CH10_DBG_TCR":                 0xa84|UINT32,
    "CH11_DBG_CTDREQ":              0xac0|UINT32,
    "CH11_DBG_TCR":                 0xac4|UINT32
    }


#Data Request signals as TREQ
DREQ_PIO0_TX0 = 0
DREQ_PIO0_TX1 = 1
DREQ_PIO0_TX2 = 2
DREQ_PIO0_TX3 = 3
DREQ_PIO0_RX0 = 4
DREQ_PIO0_RX1 = 5
DREQ_PIO0_RX2 = 6
DREQ_PIO0_RX3 = 7
DREQ_PIO1_TX0 = 8
DREQ_PIO1_TX1 = 9
DREQ_PIO1_TX2 = 10
DREQ_PIO1_TX3 = 11
DREQ_PIO1_RX0 = 12
DREQ_PIO1_RX1 = 13
DREQ_PIO1_RX2 = 14
DREQ_PIO1_RX3 = 15
DREQ_SPI0_TX = 16
DREQ_SPI0_RX = 17
DREQ_SPI1_TX = 18
DREQ_SPI1_RX = 19
DREQ_UART0_TX = 20
DREQ_UART0_RX = 21
DREQ_UART1_TX = 22
DREQ_UART1_RX = 23
DREQ_PWM_WRAP0 = 24
DREQ_PWM_WRAP1 = 25
DREQ_PWM_WRAP2 = 26
DREQ_PWM_WRAP3 = 27
DREQ_PWM_WRAP4 = 28
DREQ_PWM_WRAP5 = 29
DREQ_PWM_WRAP6 = 30
DREQ_PWM_WRAP7 = 31
DREQ_I2C0_TX = 32
DREQ_I2C0_RX = 33
DREQ_I2C1_TX = 34
DREQ_I2C1_RX = 35
DREQ_ADC = 36
DREQ_XIP_STREAM = 37
DREQ_XIP_SSITX = 38
DREQ_XIP_SSIRX = 39
#Transfer Request signals
TREQ_TIMER0 = 0x3b
TREQ_TIMER1 = 0x3c
TREQ_TIMER2 = 0x3d
TREQ_TIMER3 = 0x3e
TREQ_PERMANENT = 0x3f

# used in CTRL_TRIG.DATA_SIZE:
SIZE_BYTE = 0
SIZE_HALFWORD = 1
SIZE_WORD = 2

DMA_CHANNELS = [struct(DMA_BASE + n*0x40, DMA_CHANNEL_REGS) for n in range(12)]
DMA_DEVICE = struct(DMA_BASE, DMA_REGS)